<?php

namespace App\Http\Controllers;

use App\Event;
use App\News;
use App\Notification;
use App\UserEvent;
use App\User;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;


class EventController extends Controller
{
    public function __construct()
    {
        $this->middleware([ "auth" ]);
        $this->middleware([ "lang" ]);

        $this->middleware([ "check.auth:event.list" ])->only("index");
        $this->middleware([ "check.auth:event.show" ])->only("show");
        $this->middleware([ "check.auth:event.create" ])->only("create", "store");
        $this->middleware([ "check.auth:event.edit" ])->only("edit", "update");
        $this->middleware([ "check.auth:event.delete" ])->only("delete");
    }

    /**
     * Display a listing of the resource..
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function index(Request $request)
    {
        DB::table('user_events')
            ->where('event_id')
            ->delete();
        DB::table('events')
            ->where('events.date', '<=', date('Y-m-d H:i', strtotime('-1 day')))
            ->delete();

        $events = Event::query()->orderBY('date' , 'asc')->paginate($request->input("limit", 20));

        //returns the function with events index page and a parameter of events.
        //also Response::detect checks screen size to determine if user is on a computer or mobile.
        return Response::detect("events.index", [ "events" => $events]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function create()
    {
        //returns "create event" blade
        return Response::detect("events.create");
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function store(Request $request)
    {
        $requestBody = $request->validate([
            "name" => "required|max:255",
            "accountable" => "required|max:255",
            "description" => "required",
            "date" => "required"
        ]);

        //creates a new Event model with the given parameter
        $event = new Event($requestBody);

        $allEvents = Event::query()->where('name', '=', $request->name)->get();

        if(count($allEvents) > 0) {
            return redirect()->route("events.index", ['events' => $event]);
        } else {
            if($request->file("resource")) {
                $event->resource_id = ResourceController::store($request)->id;
            }

            $event->save();
            $events = Event::query()->paginate($request->input("limit", 20));

            if($request->newsoption == true){
                $news = new News();

                $news->name = "Ny aktivitet";
                $news->subname = $event->name;
                $news->arrangement_id = $event->id;
                $news->type_id = '3';
                $news->content = $event->description;
                $news->resource_id = $event->resource_id;

                NewsController::storeAndGet($news);
            }

            return redirect()->route('events.index', ['events' => $events]);
        }

    }

    /**
     * Display the specified resource.
     *
     * @param Event $id
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function show($id)
    {
        $event = Event::query()->where("id", "=", $id)->first();
        return Response::detect("events.show", [ "event" => $event ]);
    }

    /**
     * Display signups for event.
     *
     * @param  int  $id
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function signups(Request $request)
    {
        // Find every event you have clicked on. And find all users to that event, and the event name itself.
        $events = UserEvent::join('users', 'users.id', '=', 'user_events.user_id')->join('events', 'events.id', '=', 'user_events.event_id')->where('event_id', $request->event)->get();

        if (count($events) == 0)
            $events = Event::where('id', $request->event)->get();

        return Response::detect("events.signups", [ "events" => $events ]);
    }

    /**
     * Display signups for event.
     *
     * @param  int  $id
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function accountsignups(Request $request)
    {
        // Find every event you have clicked on. And find all users to that event, and the event name itself. - This is only being showed on App
        $events = UserEvent::join('users', 'users.id', '=', 'user_events.user_id')->join('events', 'events.id', '=', 'user_events.event_id')->where('event_id', $request->event)->get();

        return Response::detect("events.signups", [ "events" => $events ]);
    }

    /**
     * Show the form for editing the specified resource.-
     *
     * @param  int  $id
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function edit($id)
    {
        $event = Event::find($id);
        return Response::detect("events.edit", [ "event" => $event ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function update(Request $request, $id)
    {
        $data = $request->all();

        $event = Event::find($id);

        $allEvents = Event::query()->where('name', '=', $request->name)->where('id', '!=', $id)->get();

        if(count($allEvents) > 0){
            return redirect()->route("events.index", ['events' => $event]);
        }else{
            $event->update($data);
            $event->save();

            $events = Event::query()->paginate($request->input("limit", 20));

            if($request->newsoption == true){
                $news = new News();

                $news->name = "Aktivitet opdateret";
                $news->subname = $event->name;
                $news->arrangement_id = $event->id;
                $news->type_id = '3';
                $news->content = "<p>" . $this->closetags(substr($event->description, 0, 300)). "</p>" .
                    '<div class="row" style="justify-content: space-between;">'. //route("userevents.createajax", ["event_id" => $event->id ])
                    '<a style="margin: 0; padding: 0; font-weight: 700;" class="sde-blue" href="javascript:void(0);" onclick="ajaxCall('. $event->id .')">Tilmeld</a>'.
                    '<a style="margin: 0; padding: 0; text-align: center; font-weight: 700;" class="sde-blue" href="'. route("events.show", ["event" => $event->id ]) .'">Læs mere</a>'.
                    '<a style="margin: 0; padding: 0; text-align: end; font-weight: 700;" class="sde-blue" href="'. route("events.accountsignups", ["event" => $event->id ]) .'">Se deltagere</a>'.
                    '</div>';

                NewsController::storeAndGet($news);
            }

            return redirect()->route("events.index", ['events' => $events]);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Event $id
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\View\View
     * @throws \Exception
     */
    public function destroy(Request $request, $id)
    {
        if ($request->signup != null) { // If input signup is not empty, which has been set when you look at the individuel signup, then delete the user who have signed up for the event
            $UserEvent = UserEvent::query()->where('user_id', "=", $request->signup)->where('event_id', "=", $id);

            $UserEvent->delete();

            return redirect()->route("events.signups", [ "event" => $id ]);
        } else { // Else if you are deleting an event. Then delete all the sign ups AND the event
            $userEvents = UserEvent::query()->where('event_id', "=", $id)->get();
            $event = Event::query()->find($id);

            foreach ($userEvents as $userEvent) {
                $notification = new Notification();

                $notification->user_id = $userEvent->user_id;
                $notification->message = $event->name . " - ";
                $notification->save();
                $userEvent->delete();
            }

            $news = News::query()->where("type_id", "=", "3")->where("arrangement_id", "=", $event->id)->first();
            if($news !== null)
                $news->delete();

            $event->delete();

            return redirect()->route("events.index");
        }
    }

    public function search(Request $request){
        if($request->ajax()){
            $output = "<tr>".
                "<th>Aktivitet Navn</th>".
                "<th>Aktivitet Ansvarlig</th>".
                "<th>Aktivitet Dato</th>".
                "<th style=\"width: 1em;\"><img class=\"w-100\" src=".asset('/images/icons/eye.svg')." alt=\"see\"></th>".
                "<th style=\"width: 1em;\"><img class=\"w-100\" src=".asset('/images/icons/pencil.svg')." alt=\"Update\"></th>".
                "<th style=\"width: 1em;\"><img class=\"w-100\" src=".asset('/images/icons/trashcan.svg')." alt=\"Delete\"></th>".
                "</tr>";


            $events = DB::table('events')->orderBY('date' , 'asc')->where('name', 'LIKE',$request->search.'%')
                ->orWhere('date','LIKE', $request->search.'%')
                ->orWhere('accountable','LIKE', $request->search.'%')
                ->get();//Job is here dude :9

            if(count($events) !== 0){
                foreach ($events as $event){
                    if(date('Y-m-d H:i', strtotime('-1 day')) < date('Y-m-d H:i', strtotime($event->date))) {
                        $output .= '<tr>' .
                            '<td>' . $event->name . '</td>' .
                            '<td>' . $event->accountable . '</td>' .
                            '<td>' . \Illuminate\Support\Facades\Date::createFromTimeStamp(strtotime($event->date))->format('d/m/Y \k\l\. H:i') . '</td>' .
                            '<td><a href="' . route("events.signups", ["event" => $event->id]) . '"><img class="w-100" src="' . asset('/images/icons/eye-dark.svg') . '" alt="Update"></a></td>' .
                            '<td><a href="' . route("events.edit", ["event" => $event->id]) . '"><img class="w-100" src="' . asset('/images/icons/pencil-dark.svg') . '" alt="Update"></a></td>' .
                            '<td><form method="post" action="' . route("events.destroy", ["event" => $event->id]) . '" class="w-100 nostyle">' .
                            csrf_field() .
                            method_field("delete") .

                            '<button class="w-100 nostyle" onclick="return confirm(\'Are you sure you want to delete?\');" type="submit"><img class="w-100 cursor-pointer" src="' . asset('/images/icons/trashcan-dark.svg') . '" alt="Delete"></button>' .
                            '</form>' .
                            '</td>' .
                            '</tr>';
                    }
                }
            }else {
                $output.='<tr>'.
                    '<td>Intet match</td>'.
                    '<td></td>'.
                    '<td></td>'.
                    '<td></td>'.
                    '<td></td>'.
                    '<td></td>'.
                    '</tr>';
            }
            return Response($output);
        }
    }

    public function closetags($html) {
        preg_match_all('#<(?!meta|img|br|hr|input\b)\b([a-z]+)(?: .*)?(?<![/|/ ])>#iU', $html, $result);
        $openedtags = $result[1];
        preg_match_all('#</([a-z]+)>#iU', $html, $result);
        $closedtags = $result[1];
        $len_opened = count($openedtags);
        if (count($closedtags) == $len_opened) {
            return $html;
        }
        $openedtags = array_reverse($openedtags);
        for ($i=0; $i < $len_opened; $i++) {
            if (!in_array($openedtags[$i], $closedtags)) {
                $html .= '</'.$openedtags[$i].'>';
            } else {
                unset($closedtags[array_search($openedtags[$i], $closedtags)]);
            }
        }
        return $html;
    }

    public function deleteNotifications(Request $request){
        if($request->ajax()){
            foreach (Notification::query()->where("user_id", "=", $request->user_id)->get() as $notification) {
                $notification->delete();
            }
            return "Done";
        } else {
            return "ERROR";
        }
    }

    public function nameCheck(Request $request){
        $event = Event::query()->where('name', 'LIKE',$request->nameCheck)->get();
        if(count($event) > 0 && $request->nameCheck !== NULL){
            return 1;
        }
    }

    public function nameCheckUpdate(Request $request){
        $event = Event::query()->where('name', 'LIKE',$request->nameCheck)->where('id', '!=', $request->id)->get();
        if(count($event) > 0 && $request->nameCheck !== NULL){
            return 1;
        }
    }

}


