<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;
use Spatie\Permission\Models\Role;

class rolesController extends Controller
{
    public function __construct()
    {
        $this->middleware([ "auth" ]);
        $this->middleware([ "lang" ]);

        $this->middleware([ "check.auth:roles.list" ])->only("index");
        $this->middleware([ "check.auth:roles.show" ])->only("show");
        $this->middleware([ "check.auth:roles.create" ])->only("create", "store");
        $this->middleware([ "check.auth:roles.edit" ])->only("edit", "update");
        $this->middleware([ "check.auth:roles.delete" ])->only("delete");
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $roles = Role::query()->paginate($request->input("limit", 20));

        return Response::detect("roles.index", [ "roles" => $roles]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return Response::detect("roles.create");

    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $requestRole = $request->validate([
            "name" => "required|max:255",
            "description" => "required|max:255"
        ]);

        $role = new Role($requestRole);

        $allRoles = Role::query()->where('name', '=', $request->name)->get();

        // If there already is a role with that name, then don't create
        if(count($allRoles) > 0)
            return redirect()->route("roles.store");
        else { // Else - Create it
            $role->save();
            $roles = Role::query()->paginate($request->input("limit", 20));
            return redirect()->route("roles.index", ['roles' => $roles]);
        }


    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        return Response::detect("roles.show", [ "role" => $id]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $role = Role::find($id);
        return Response::detect("roles.edit", ["role" => $role]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $data = $request->all();
        $role = Role::find($id);

        $allRoles = Role::query()->where('name', '=', $request->name)->where('id', '!=', $id)->get();

        if(count($allRoles) > 0){
            return redirect()->route("roles.index", [ "role" => $role ]);
        }else{
            $role->update($data);
            $role->save();
            $roles = Role::query()->paginate($request->input("limit", 20));
            return redirect()->route("roles.index", ['roles' => $roles]);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $role = Role::find($id);
        $role->delete();
        return redirect()->route("roles.index");
    }


    public function search(Request $request){
        if($request->ajax()){
            $output = "<tr>".
                "<th>Navn</th>".
                "<th>Beskrivelse</th>".
                "<th style=\"width: 1em;\"><img class=\"w-100\" src=".asset('/images/icons/pencil.svg')." alt=\"Update\"></th>".
                "<th style=\"width: 1em;\"><img class=\"w-100\" src=".asset('/images/icons/trashcan.svg')." alt=\"Delete\"></th>".
                "</tr>";
            $roles = DB::table('roles')->where('name', 'LIKE',$request->search.'%')->get();

            if(count($roles) !== 0){
                foreach ($roles as $key => $role){
                    $output.='<tr>'.
                        '<td>' . $role->name . '</td>'.
                        '<td>' . $role->description . '</td>'.
                        '<td><a href="'. route("roles.edit", [ "role" => $role->id ]) . '"><img class="w-100" src="'. asset('/images/icons/pencil-dark.svg') . '" alt="Update"></a></td>'.
                        '<td><form method="post" action="' .route("roles.destroy", [ "role" => $role->id ]). '" class="w-100 nostyle">'.
                        csrf_field().
                        method_field("delete").

                        '<button class="w-100 nostyle" onclick="return confirm(\'Are you sure you want to delete?\');" type="submit"><img class="w-100 cursor-pointer" src="'. asset('/images/icons/trashcan-dark.svg') . '" alt="Delete"></button>'.
                        '</form>'.
                        '</td>'.
                        '</tr>';
                }
            }else{
                $output.='<tr>'.
                    '<td>Intet match</td>'.
                    '<td></td>'.
                    '<td></td>'.
                    '<td></td>'.
                    '</tr>';
            }
            return Response($output);
        }
    }

    public function nameCheck(Request $request){
        $roles = Role::query()->where('name', 'LIKE',$request->nameCheck)->get();
        if(count($roles) > 0 && $request->nameCheck !== NULL){
            return 1;
        }
    }

    public function nameCheckUpdate(Request $request){
        $roles = Role::query()->where('name', 'LIKE',$request->nameCheck)->where('id', '!=', $request->id)->get();
        if(count($roles) > 0 && $request->nameCheck !== NULL){
            return 1;
        }
    }
}
