<?php

namespace App\Http\Controllers;

use App\Location;
use App\WashingReservation;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\View\Factory;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

use App\WashingMachine;
use Illuminate\Support\Facades\DB;
use Illuminate\View\View;

class WashingMachineController extends Controller
{
    public function __construct()
    {
        $this->middleware([ "auth" ]);
        $this->middleware([ "lang" ]);

        $this->middleware([ "check.auth:washing.machine.list" ])->only("index");
        $this->middleware([ "check.auth:washing.machine.show" ])->only("show");
        $this->middleware([ "check.auth:washing.machine.create" ])->only("create", "store");
        $this->middleware([ "check.auth:washing.machine.edit" ])->only("edit", "update");
        $this->middleware([ "check.auth:washing.machine.delete" ])->only("delete");
    }

    /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return Application|Factory|View
     */
    public function index(Request $request)
    {
        $machines = WashingMachine::query()->paginate($request->query("limit", 20));

        return Response::detect("washing-machines.index", [ "machines" => $machines ]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return Application|Factory|View
     */
    public function create()
    {
        $locations = Location::all();

        return Response::detect("washing-machines.create", ["locations" => $locations] );
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return Application|Factory|View
     */
    public function store(Request $request)
    {
        $data = $request->validate([
            "name" => "required",
            "location_id" => "required"
        ]);

        $machine = new WashingMachine($data);

        $allMachines = WashingMachine::query()->where('name', '=', $request->name)->where('location_id', "=", $request->location_id)->get();

        // If there already is a washing machine with that name, then don't add it
        if (count($allMachines) > 0)
            return redirect()->route("washing-machines.store");
        else { // Else - Add it
            $machine->save();
            $machines = WashingMachine::query()->paginate($request->input("limit", 20));
            return redirect()->route("washing-machines.index", ['machines' => $machines]);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return Application|Factory|View
     */
    public function show($id)
    {

    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return Application|Factory|View
     */
    public function edit($id)
    {
        $machine = WashingMachine::find($id);
        $locations = Location::all();

        return Response::detect("washing-machines.edit", [
            "machine" => $machine,
            "locations" => $locations
        ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return Application|Factory|View
     */
    public function update(Request $request, $id)
    {
        $data = $request->validate([
            "name" => "required",
            "location_id" => "required",
        ]);

        $machine = WashingMachine::find($id);


        $allMachines = WashingMachine::query()->where('name', '=', $request->name)->where('location_id', "=", $request->location_id)->where('id', '!=', $id)->get();

        // If there already is a washing machine with that name, then don't change it
        if (count($allMachines) > 0)
            return redirect()->route("washing-machines.store");
        else { // Else - Change the name
            $machine->update($data);
            $machine->save();
            $machines = WashingMachine::query()->paginate($request->input("limit", 20));
            return redirect()->route("washing-machines.index", ["machines" => $machines]);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param $id
     * @return Response
     */
    public function destroy($id)
    {
        $machine = WashingMachine::find($id);

        $allReservations = WashingReservation::query()->where('machine_id', '=', $id);

        $allReservations->delete();
        $machine->delete();

        return redirect()->route("washing-machines.index");
    }

    public function nameCheck(Request $request){
        $washing = WashingMachine::query()->where('name', 'LIKE',$request->nameCheck)->where('location_id', '=', $request->location)->get();
        if(count($washing) > 0 && $request->nameCheck !== NULL){
            return 1;
        }
    }

    public function nameCheckUpdate(Request $request){
        $washing = WashingMachine::query()->where('name', 'LIKE',$request->nameCheck)->where('location_id', '=', $request->location)->where('id', '!=', $request->id)->get();
        if(count($washing) > 0 && $request->nameCheck !== NULL){
            return 1;
        }
    }

    public function search(Request $request){
        if($request->ajax()){
            $output = "<tr>".
                "<th>Navn</th>".
                "<th>Lokation</th>".
                "<th style=\"width: 1em;\"><img class=\"w-100\" src=".asset('/images/icons/pencil.svg')." alt=\"Update\"></th>".
                "<th style=\"width: 1em;\"><img class=\"w-100\" src=".asset('/images/icons/trashcan.svg')." alt=\"Delete\"></th>".
                "</tr>";

            if($request->option !== 'all') {
                $machines = DB::table('washing_machines')
                    ->join('locations', 'washing_machines.location_id','=', 'locations.id')
                    ->select(DB::raw('washing_machines.name as washing_name, locations.name as location_name, washing_machines.id as id'))
                    ->where('locations.id', '=', $request->option)
                    ->get();

                if(count($machines) !== 0){
                    foreach ($machines as $machine){
                        $output.='<tr>'.
                            '<td>' . $machine->washing_name . '</td>'.
                            '<td>' . $machine->location_name . '</td>'.
                            '<td><a href="'. route("washing-machines.edit", [ "washing_machine" => $machine->id ]) . '"><img class="w-100" src="'. asset('/images/icons/pencil-dark.svg') . '" alt="Update"></a></td>'.
                            '<td><form method="post" action="' .route("washing-machines.destroy", [ "washing_machine" => $machine->id ]). '" class="w-100 nostyle">'.
                            csrf_field().
                            method_field("delete").

                            '<button class="w-100 nostyle" onclick="return confirm(\'Are you sure you want to delete?\');" type="submit"><img class="w-100 cursor-pointer" src="'. asset('/images/icons/trashcan-dark.svg') . '" alt="Delete"></button>'.
                            '</form>'.
                            '</td>'.
                            '</tr>';
                    }
                }else{
                    $output.='<tr>'.
                        '<td>Det er ikke blivet oprettet nogen vaskmaskine på denne lokation endnu</td>'.
                        '<td></td>'.
                        '<td></td>'.
                        '<td></td>'.
                        '</tr>';
                }
            }
            elseif($request->option === "all"){
                $machines = WashingMachine::all();

                if(count($machines) !== 0){
                    foreach($machines as $machine) {
                        $output.= '<tr>'.
                            '<td>'.$machine->name.'</td>'.
                            '<td>'.Location::query()->where("id", "=", $machine->location_id)->first()->name.'</td>'.
                            '<td><a href="'. route("washing-machines.edit", [ "washing_machine" => $machine->id]) . '"><img class="w-100" src="'. asset('/images/icons/pencil-dark.svg') . '" alt="Update"></a></td>'.
                            '<td><form method="post" action="'. route('washing-machines.destroy', [ 'washing_machine' => $machine->id ]) .'" class="w-100 nostyle">'.
                            csrf_field().
                            method_field("delete").
                            '<button class="w-100 nostyle" onclick="return confirm('."'Are you sure you want to delete?'".');" type="submit"><img class="w-100 cursor-pointer" src="'. asset('/images/icons/trashcan-dark.svg'). '" alt="Delete"></button>'.
                            '</form>'.
                            '</td>'.
                            '</tr>';
                    }
                }else{
                    $output.='<tr>'.
                        '<td>Det er ikke blivet oprettet nogen vaskmaskine på denne lokation endnu</td>'.
                        '<td></td>'.
                        '<td></td>'.
                        '<td></td>'.
                        '</tr>';
                }
            }




            return Response($output);
        }
    }
}
